/* eslint-disable no-magic-numbers */
import { Feature } from 'ol';
import VectorLayer from 'ol/layer/Vector';
import VectorSource from 'ol/source/Vector';
import { useEffect, useMemo } from 'react';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import CustomMultiPolygon from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/CustomMultiPolygon';
import { useSelector } from 'react-redux';
import { shapesSelector } from '@/redux/shapes/shapes.selectors';
import { MapInstance } from '@/types/map';
import {
  HorizontalAlign,
  VerticalAlign,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.types';
import { modelElementsSelector } from '@/redux/modelElements/modelElements.selector';
import { currentModelIdSelector } from '@/redux/models/models.selectors';
import { getModelElements } from '@/redux/modelElements/modelElements.thunks';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';

export const useOlMapReactionsLayer = ({
  mapInstance,
}: {
  mapInstance: MapInstance;
}): VectorLayer<VectorSource<Feature>> => {
  const dispatch = useAppDispatch();
  const modelElements = useSelector(modelElementsSelector);
  const currentModelId = useSelector(currentModelIdSelector);
  useEffect(() => {
    dispatch(getModelElements(currentModelId));
  }, [currentModelId, dispatch]);

  const pointToProjection = usePointToProjection();
  const shapes = useSelector(shapesSelector);

  const elements = useMemo(() => {
    if (modelElements) {
      return modelElements.content.map(element => {
        const shape = shapes.data.find(bioShape => bioShape.sboTerm === element.sboTerm);
        if (shape) {
          return new CustomMultiPolygon({
            shapes: shape.shapes,
            x: element.x,
            y: element.y,
            nameX: element.nameX,
            nameY: element.nameY,
            nameHeight: element.nameHeight,
            nameWidth: element.nameWidth,
            width: element.width,
            height: element.height,
            zIndex: element.z,
            lineWidth: element.lineWidth,
            fontColor: element.fontColor,
            fillColor: element.fillColor,
            borderColor: element.borderColor,
            nameVerticalAlign: element.nameVerticalAlign as VerticalAlign,
            nameHorizontalAlign: element.nameHorizontalAlign as HorizontalAlign,
            text: element.name,
            pointToProjection,
            mapInstance,
          });
        }
        return undefined;
      });
    }
    return [];
  }, [mapInstance, pointToProjection, shapes.data, modelElements]);

  const features = useMemo(() => {
    return elements
      .filter((element): element is CustomMultiPolygon => element !== undefined)
      .map(element => element.multiPolygonFeature);
  }, [elements]);

  const vectorSource = useMemo(() => {
    return new VectorSource({
      features,
    });
  }, [features]);

  return useMemo(
    () =>
      new VectorLayer({
        source: vectorSource,
      }),
    [vectorSource],
  );
};
