import getPolygonCoords from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/getPolygonCoords';
import getEllipseCoords from '@/components/Map/MapViewer/MapViewerVector/utils/shapes/getEllipseCoords';
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import Polygon from 'ol/geom/Polygon';
import { Coordinate } from 'ol/coordinate';
import { Shape } from '@/types/models';

export default function getMultiPolygon({
  x,
  y,
  width,
  height,
  shapes,
  pointToProjection,
}: {
  x: number;
  y: number;
  width: number;
  height: number;
  shapes: Array<Shape>;
  pointToProjection: UsePointToProjectionResult;
}): Array<Polygon> {
  return shapes.map(shape => {
    let coords: Array<Coordinate> = [];
    if (shape.type === 'POLYGON') {
      coords = getPolygonCoords({ points: shape.points, x, y, height, width, pointToProjection });
    } else if (shape.type === 'ELLIPSE') {
      coords = getEllipseCoords({
        x,
        y,
        center: shape.center,
        radius: shape.radius,
        height,
        width,
        pointToProjection,
      });
    }
    return new Polygon([coords]);
  });
}
