/* eslint-disable no-magic-numbers */
import { Fill, Text } from 'ol/style';
import Style from 'ol/style/Style';
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import { Feature } from 'ol';
import { Point } from 'ol/geom';
import { Coordinate } from 'ol/coordinate';
import {
  HorizontalAlign,
  VerticalAlign,
} from '@/components/Map/MapViewer/MapViewerVector/MapViewerVector.types';

export default function getText({
  x,
  y,
  height,
  width,
  text = '',
  fontSize = 12,
  color = '#000',
  verticalAlign = 'MIDDLE',
  horizontalAlign = 'CENTER',
  pointToProjection,
}: {
  x: number;
  y: number;
  height: number;
  width: number;
  text: string;
  fontSize?: string | number;
  color?: string;
  verticalAlign?: VerticalAlign;
  horizontalAlign?: HorizontalAlign;
  pointToProjection: UsePointToProjectionResult;
}): { textCoords: Coordinate; textStyle: Style } {
  const minX = x;
  const maxX = x + width;
  const minY = y;
  const maxY = y + height;

  let textY = (minY + maxY) / 2;
  if (verticalAlign === 'TOP') {
    textY = minY + +fontSize / 2;
  } else if (verticalAlign === 'BOTTOM') {
    textY = maxY - +fontSize / 2;
  }

  let textX = (minX + maxX) / 2;
  if (['LEFT', 'START'].includes(horizontalAlign)) {
    textX = minX;
  } else if (['RIGHT', 'END'].includes(horizontalAlign)) {
    textX = maxX;
  }

  const textCoords = pointToProjection({ x: textX, y: textY });
  const textFeature = new Feature({ geometry: new Point(textCoords) });

  const textStyle = new Style({
    text: new Text({
      text,
      font: `bold ${fontSize}px Arial`,
      fill: new Fill({
        color,
      }),
      placement: 'point',
      textAlign: 'center',
      textBaseline: 'middle',
    }),
  });
  textFeature.setStyle(textStyle);

  return { textCoords, textStyle };
}
